@@ -1,11 +1,15 @@
 from ._python_calamine import (
     CalamineError,
     CalamineSheet,
+    CalamineTable,
     CalamineWorkbook,
     PasswordError,
     SheetMetadata,
     SheetTypeEnum,
     SheetVisibleEnum,
+    TableNotFound,
+    TablesNotLoaded,
+    TablesNotSupported,
     WorkbookClosed,
     WorksheetNotFound,
     XmlError,
@@ -16,14 +20,18 @@ from ._python_calamine import (
 __all__ = (
     "CalamineError",
     "CalamineSheet",
+    "CalamineTable",
     "CalamineWorkbook",
     "PasswordError",
     "SheetMetadata",
     "SheetTypeEnum",
     "SheetVisibleEnum",
+    "TableNotFound",
+    "TablesNotLoaded",
+    "TablesNotSupported",
+    "WorkbookClosed",
     "WorksheetNotFound",
     "XmlError",
     "ZipError",
-    "WorkbookClosed",
     "load_workbook",
 )
