@@ -1,10 +1,13 @@
 mod cell;
 mod errors;
 mod sheet;
+mod table;
 mod workbook;
 pub use cell::CellValue;
 pub use errors::{
-    CalamineError, Error, PasswordError, WorkbookClosed, WorksheetNotFound, XmlError, ZipError,
+    CalamineError, Error, PasswordError, TableNotFound, TablesNotLoaded, TablesNotSupported,
+    WorkbookClosed, WorksheetNotFound, XmlError, ZipError,
 };
 pub use sheet::{CalamineSheet, SheetMetadata, SheetTypeEnum, SheetVisibleEnum};
+pub use table::CalamineTable;
 pub use workbook::CalamineWorkbook;
